from typing import Any, Dict

import httpx

from ...client import Client
from ...types import Response


def _get_kwargs(
    bot_user_id: str,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/bots/{bot_user_id}/icon".format(client.base_url, bot_user_id=bot_user_id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    bot_user_id: str,
    *,
    client: Client,
) -> Response[Any]:
    """Get bot's LHS icon

     Get a bot's LHS icon image based on bot_user_id string parameter.
    ##### Permissions
    Must be logged in.
    __Minimum server version__: 5.14

    Args:
        bot_user_id (str):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        bot_user_id=bot_user_id,
        client=client,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    bot_user_id: str,
    *,
    client: Client,
) -> Response[Any]:
    """Get bot's LHS icon

     Get a bot's LHS icon image based on bot_user_id string parameter.
    ##### Permissions
    Must be logged in.
    __Minimum server version__: 5.14

    Args:
        bot_user_id (str):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        bot_user_id=bot_user_id,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)
